
# Created by Amin Rahmani, Ph.D., P.E. - February 14, 2018
# Copyright © 2018 Amin Rahmani. All rights reserved.

#========================================================================================
#                               ****   Notes  ****
# This code performs seismic analysis for a ground improvement (GI) element such as  stone column, Deep Soil Mixing column, Grouted Piers, etc embedded in saturated soil continuum domain.
# The code creates a unit cell which comprises a single GI element and its equivalent influence zone. Influence zone is considered to be the center to center spacing of the elements.
# You need to makes changes on the lines that the comment "User-Defined" appears. Please go through all the command lines before running it.
# The code creates a half mesh. The model is symmetric in the direction perpendicular to shaking.
# Units are Imperial (lbf, ft, sec). Check the units if you work with Metric units. 
# The input motions should be available in two separate .txt files. Time in "SCTime.txt" and Displacement in "SCDisp.txt".
# When compliant boundary condition is used, the input motions should be available in two separate .txt files. Time in "Time.txt" and Velocity in "Velocity.txt".
# Outputs will be saved in a folder titled "Outputs".
#========================================================================================

wipe all
set startT [clock seconds]

##### This will create an input file for GID program if you want to check the final geometry.
set meshFile [open soilmesh.flavia.msh w]
#############################################################################################


#######################################################################
#### Input parameters used for creating the geometry of soil-pile model 
#######################################################################

set gravX  0.0000
set gravY  0.0000
set gravZ  -32.2 				;# gravity in direction Z. The value would be -32.2 if Imperial units are used.

set BP       3.5                ;# User-Defined --- Ground Improvement (GI) Element Width                                         
set LP       16.0               ;# User-Defined --- GI Element Length  ** Note: Zone meshing in vertical direction should be consistent with the pile length.
						        ;# zo and z1 defined below should be set in a way that LP >= z0+z1
set HZ       16.0               ;# User-Defined --- Soil Deposit Height

#Unit cell 
set S    11.0					;# User-Defined --- Unit cell width in direction X
set S_Y  10.0				    ;# User-Defined --- Unit cell width in direction Y

set WInf       [expr ($S/2.0-$BP/2)-int($S/2.0-$BP/2)] 
set WInf_Y     [expr ($S_Y/2.0-$BP/2)-int($S_Y/2.0-$BP/2)] 
if {$WInf <= 0.0 || $WInf_Y <= 0.0} {
   break
   }

set xSize1   1.0                 ;# User-Defined --- Element size in direction X 
set ySize1   1.0                 ;# User-Defined --- Element size in direction Y

set LX       $S                  ;# Model length in Direction X      
set BY       $S_Y                ;# Model length in Direction Y        

#### You can apply 3 levels of mesh sizes in the vertical direction, z0, z1, and z2 represent layer thichness at the top, middle, and bottom layers, respectively.
set z0  10.0  					; # User-Defined ---  Thickness of zone 0
set z1  3.0    					; # User-Defined ---  Thickness of zone 1
								; # zo and z1 should be set in a way that LP >= z0+z1
set z2  3.0    					; # User-Defined ---  Thickness of zone 2

set zSize0    1.0               ;# User-Defined ---  z direction element size 0
set zSize1    1.0               ;# User-Defined ---  z direction element size 1
set zSize2    1.0               ;# User-Defined ---  z direction element size 2

puts "$LX $BY"
set numXele  [expr int((($S/2.0-$BP/2.0-$WInf)/$xSize1)*2+2+4) ]          ;# number of elements in x  direction 
set numYele  [expr int((($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1)*2+2+4)]       ;# number of elements in y  direction 
set numZele  [expr int($z0/$zSize0+$z1/$zSize1+$z2/$zSize2)]              ;# number of elements in z  direction    direction

set numXnode  [expr $numXele+1]
set numYnode  [expr $numYele/2+1]
set numZnode  [expr $numZele+1]
set ele_sp    [expr $numZnode-int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2)]

###########################################################
#### Creating Nodes for 3-dimensional single column of soil
###########################################################

model basic -ndm 3 -ndf 4

#### For GID Program Use
puts $meshFile "MESH stdBrick dimension 3 ElemType Hexahedra Nnode 8"
puts $meshFile "Coordinates"
########################

set m 0
for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {
    for {set k 1} {$k <= $numZnode} {incr k 1} {

      ### nodes xdim 
	  if { $i <= [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 1]  } {
	     set xdim [expr ($i-1)*$xSize1]
		 } else {
	  if { $i == [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 2]   } {
	     set xdim [expr $S/2.0-$BP/2.0]
		 } else {
	  if { $i > [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 2]  && $i <= [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 6] } {
	     set xdim [expr $S/2.0-$BP/2.0+($i-(($S/2.0-$BP/2.0-$WInf)/$xSize1 + 2))*$BP/4.0]	  
		 } else {
	  if { $i == [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 7]   } {
	     set xdim [expr $S/2.0+$BP/2.0+$WInf]	
		 } else {
         set xdim [expr $S/2.0+$BP/2.0+$WInf + ($i-(($S/2.0-$BP/2.0-$WInf)/$xSize1 + 7))*$xSize1]
		 }}}}
		 
      ### nodes ydim 
	  if { $j <= [expr ($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1 + 1]  } {
	     set ydim [expr ($j-1)*$ySize1]
		 } else {
	  if { $j == [expr ($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1 + 2]   } {
	     set ydim [expr $S_Y/2.0-$BP/2.0]
		 } else {
	  if { $j == [expr ($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1 + 3]   } {
	     set ydim [expr $S_Y/2.0-$BP/4.0]	  
		 } else {
           set ydim [expr $S_Y/2.0]	
             }}}

      ### nodes zdim 
      if { $k <= [expr ($z2/$zSize2+1)] } {
          set   zdim [expr ($k-1)*$zSize2]
      } else {
         if { $k > [expr ($z2/$zSize2)+($z1/$zSize1)+1] } {
          set   zdim [expr $z2+$z1+($k-(($z2/$zSize2)+($z1/$zSize1)+1))*$zSize0]
      } else {
          set   zdim [expr $z2+($k-(($z2/$zSize2)+1))*$zSize1]
      }}
	  
      set   nodeNum   [expr $m*$numZnode+$k] 
      node  $nodeNum  $xdim $ydim $zdim
	  
	  #### For GID Program Use
      puts $meshFile "$nodeNum  $xdim $ydim $zdim"

   ###### Specifying Node numbers in center and edge of GI Element
   if { $i == [expr $numXele/2-1] && $j == [ expr $numYele/2-1 ] && $k == $ele_sp } {
    set p1 $nodeNum
    puts "p1=$p1"
   } else {
   if { $i == [expr $numXele/2] && $j == [ expr $numYele/2-1 ] && $k == $ele_sp  } {
    set p2 $nodeNum
    puts "p2=$p2"
   } else {
   if { $i == [expr $numXele/2+1] && $j == [ expr $numYele/2-1 ] && $k == $ele_sp  } {
    set p3 $nodeNum
    puts "p3=$p3"
   } else {
   if { $i == [expr $numXele/2+2] && $j == [ expr $numYele/2-1 ] && $k == $ele_sp  } {
    set p4 $nodeNum
    puts "p4=$p4"
   } else {
   if { $i == [expr $numXele/2+3] && $j == [ expr $numYele/2-1 ] && $k == $ele_sp  } {
    set p5 $nodeNum
    puts "p5=$p5"
   }}}}}
   }
    set m [expr ($m+1)]
  } 
}

#### For GID Program Use
puts $meshFile "end coordinates"


##########################################
#### Define Soil Materials
##########################################

# User-Defined
##Mat 1
#set Gs2      522139.0
set Gs2      353376.0
set nou2     0.3
set Bs2      [expr 2.0*$Gs2*(1+$nou2)/3.0/(1-2.0*$nou2)]
#set Bs2      1148705.1
set rhos2   [expr 126.5/32.2]
set cohes2  5.0
set fricAng2 33.0
set peakShearStra 0.10
set refPress 1671.0
set pressDependCoe 0.50
set PTAng2 25.5
set contrac 0.21
set dilat1 0.0
set dilat2 0.0
set liquefac1 208.8
set liquefac2 0.02
set liquefac3 1.0
set n2 0.409
set Perm2 [expr 5.953e-3]
set alpha2 [expr (pow($xSize1,2))/(4.0*($Bs2 + (4.0/3.0)*$Gs2))]
nDMaterial PressureDependMultiYield02 2 3 $rhos2  $Gs2 $Bs2 $fricAng2 $peakShearStra $refPress $pressDependCoe $PTAng2 0.045 0.15 0.06 0.15 20 5.0 3.0 1.0 0.0 0.7 0.9 0.02 0.7 2089.0

# User-Defined
##Mat 2
#set Gs3      730995.0
set Gs3      601503.0
set nou3     0.3
set Bs3      [expr 2.0*$Gs3*(1+$nou3)/3.0/(1-2.0*$nou3)]
#set Bs3      1587301.0
set rhos3   [expr 128.0/32.2]
set cohes3  5.0
set fricAng3 32.0
set peakShearStra 0.10
set refPress 1671.0
set pressDependCoe 0.50
set PTAng3 26.0
set contrac 0.07
set dilat1 0.60
set dilat2 3.0
set liquefac1 100.0
set liquefac2 0.003
set liquefac3 1.0
set n3 0.4000
set Perm3 [expr 3.64e-3]
set alpha3 [expr (pow($xSize1,2))/(4.0*($Bs3 + (4.0/3.0)*$Gs3))]
##puts "$alpha3"
nDMaterial PressureDependMultiYield02 3 3 $rhos3  $Gs3 $Bs3 $fricAng3 $peakShearStra $refPress $pressDependCoe $PTAng3 0.067 0.23 0.06 0.27 20 5.0 3.0 1.0 0.0 0.61  0.9 0.02 0.7 2089.0

# User-Defined
##Ground Improvement Element Material
set Gs_GI      1360.0e3
set nou_GI     0.30
set Bs_GI      [expr 2.0*$Gs_GI*(1+$nou_GI)/3.0/(1-2.0*$nou_GI)]
set rhos_GI   [expr 100.0/32.2]
set cohes_GI  1.0
set peakShearStra 0.1
set fricAng_GI  47.0
set n_GI 0.3
set Perm_GI [expr 3.0e-2]
set alpha_GI [expr (pow($xSize1,2))/(4.0*($Bs_GI + (4.0/3.0)*$Gs_GI))]
set refPress 2089.0
set pressDependCoe 0.0
set PTAng 26.0
nDMaterial PressureDependMultiYield02 999 3 $rhos_GI  $Gs_GI $Bs_GI $fricAng_GI $peakShearStra $refPress $pressDependCoe $PTAng 0.013 0.0 0.30 0.00 20 5.0 3.0 1.0 0.0 0.55  0.9 0.02 0.7 2089.0

set bulk 45950400.0 ; # User-Defined --- Bulk Modulus of Water

##########################################
#### Define Soil Elements
##########################################

#### For GID Program Use
puts $meshFile "Elements"

set m 0
for {set i 1} {$i <= $numXele} {incr i 1} {
  for {set j 1} {$j <= [expr $numYele/2.0] } {incr j 1} {
    for {set k 1} {$k <= $numZele} {incr k 1} {
      
      set eleNum [expr $m*$numZele+$k] 
      set n1  [expr ($i-1)*($numYnode*$numZnode)+$k+($j-1)*$numZnode] 
      set n2  [expr $n1+($numYnode*$numZnode)] 
      set n3  [expr $n2+$numZnode] 
      set n4  [expr $n1+$numZnode] 
      set n5  [expr $n1+1]
      set n6  [expr $n2+1]
      set n7  [expr $n3+1]
      set n8  [expr $n4+1]
	  
 # User-Defined --- Depending on the number of soil layers, you may need more "if-else" command lines. Remember to change "$K <= 5". It should represent soil stratigraphy.
 # At this stage, permeability must be set to 1000.0 to avoid pore water pressure generation during geostatic analysis. 
    if { $i >= [expr int($numXele/2.0-1)] && $i <= [expr int($numXele/2.0 + 2)] && $j >= [expr int($numYele/2.0-1)] && $j <= [expr int($numYele/2.0 )] && $k > ($LP-$z0-$z1)/$zSize2 } {
         element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 999 [expr $bulk/$n3] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]
         puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 1"
    } else {
     if { $k <= 9 } {
          element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 3  [expr $bulk/$n3] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]
		  puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 2"
        } else {
         element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8  2 [expr $bulk/$n2] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]
		 puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 3"
        }	  	   
    }
	
    }
    set m [expr ($m+1)]
  }
}

puts "Number of Nodes    =  $nodeNum"
puts "Number of Elements =  $eleNum " 

#### For GID Program Use
puts $meshFile "end elements"

close $meshFile

##############################################
#### Boundary conditions
##############################################

# Symmetry Plane
fixY [expr $S_Y/2.0] 0 1 0 0   

# Bottom Boundary 
fixZ 0.0 1 1 1 0    

# User-defined --- Ground Water Depth With Respect to Model Base 
fixZ [expr $z0+$z1+$z2] 0 0 0 1 ; #User-Defined - Use fixZ ... 0 0 0 1 for all levels above ground water table.

# Lateral Boundaries 
for {set k 2} {$k <= $numZnode} {incr k 1} {
  for {set j 2} {$j <= [expr $numYnode-1]} {incr j 1} {
      equalDOF $k [expr ($numXnode-1)*($numZnode*$numYnode)+$k+(($j-2)*$numZnode)]    1  2 3    ;# right side yz boundary 
      equalDOF $k [expr ($j-1)*$numZnode+$k]     1 2 3                                         ;# left  side yz boundary
  }
  
  equalDOF $k [expr ($numXnode*$numYnode-1)*$numZnode+$k]     1 2 3
  
  for {set i 2} {$i <=  ($numXnode)} {incr i 1} {
     equalDOF $k [expr ($i-1)*($numZnode*$numYnode)+$k]          1  2  3             ;# right side xz boundary

  }
}

##############################################
#### Elastic Geostatic Analysis
##############################################

numberer Plain
system SparseSYM
#system Mumps
test NormDispIncr 1.0e-2 100 1
algorithm KrylovNewton
constraints Penalty 1.e18 1.e18
integrator Newmark 0.5 0.25
analysis Transient 

 for {set numIncr 1} {$numIncr <= 5 } {incr numIncr 1} {
       puts "##### Elastic Geostatic Analysis step : $numIncr #####";
       analyze 1 10.0
     }

##############################################
#### Switch into Plastic State 
##############################################

for {set i 2} {$i <= 3 } {incr i 1} {
    updateMaterialStage -material $i -stage 1   ; # User-Defined --- Change Mat tag per prescribed tags
   }
updateMaterialStage -material 999 -stage 1      ; # User-Defined --- Change Mat tag per prescribed tags
 
for {set numIncr 1} {$numIncr <= 5 } {incr numIncr 1} {
      puts "##### Plastic Geostatic Analysis step : $numIncr #####";
      analyze 1 0.01
    }

##############################################
#### Update Stiffness of Materials 
##############################################	
updateMaterials -material 2 shearModulus [expr $Gs2] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 2 bulkModulus  [expr $Bs2] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

updateMaterials -material 3 shearModulus [expr $Gs3] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags 
updateMaterials -material 3 bulkModulus  [expr $Bs3]  ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

updateMaterials -material 999 shearModulus [expr $Gs_GI] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 999 bulkModulus  [expr $Bs_GI] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

##############################################
#### Preparing Model for Seismic Analysis
##############################################
wipeAnalysis
remove recorders
loadConst -time 0.0

##############################################
#### User-Defined Rayleigh Damping
##############################################

set zeta 0.05;		# User-Defined --- Damping Ratio in Decimal
set w1  21.12;		# User-Defined --- Angular Frequency at Frequency 1
set w2  14.96;		# User-Defined --- Angular Frequency at Frequency 2

set a0 [expr $zeta*2.0*$w1*$w2/($w1 + $w2)];	# mass damping coefficient based on first and second modes
set a1 [expr $zeta*2.0/($w1 + $w2)];			# stiffness damping coefficient based on first and second modes

rayleigh $a0 0.0 $a1 0.0; 


######################################################################
#### Removing Constraints at Model Base in the Direction of Excitation
######################################################################

for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {

        remove sp [expr 1+($i-1)*$numYnode*$numZnode+($j-1)*$numZnode] 1  

      }
    }

######################################################
#### Updating Soil Permeability for Seismic Shaking
######################################################


# User-Defined --- "k" values and updated permeability values should be modified to represent soil stratigraphy. It should be the same as layer numbers used in defining material to soil elements.
for {set i 1} {$i <= $numXele } {incr i 1} {
	for {set j 1} {$j <= $numYele/2 } {incr j 1} {
		for {set k 1} {$k <= $numZele} {incr k 1} {	
			
			    parameter [expr 100000+$k+($j-1)*$numZele+($i-1)*($numYele/2)*$numZele] element $i hPerm 
				parameter [expr 200000+$k+($j-1)*$numZele+($i-1)*($numYele/2)*$numZele] element $i vPerm 
	
			if { $i >= [expr int($numXele/2.0-1)] && $i <= [expr int($numXele/2.0 + 2)] && $j >= [expr int($numYele/2.0-1)] && $j <= [expr int($numYele/2.0 )] && $k > ($LP-$z0-$z1)/$zSize2 } {		
				updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele/2] $Perm_GI ; #User-Defined --- Perm_GI
				updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele/2] $Perm_GI ; #User-Defined --- Perm_GI
				} else {
				if { $k <= 9 } {
						updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele/2] [expr $Perm3]  ; #User-Defined --- Perm_3
						updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele/2] [expr $Perm3]  ; #User-Defined --- Perm_3
				} else {
						updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele/2] [expr $Perm2]  ; #User-Defined --- Perm_2
						updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele/2] [expr $Perm2]  ; #User-Defined --- Perm_2
				}								
				}
		}
	}
}

########################################################
#### Creating Output Files in a Folder titled "Outputs"
########################################################

set dataDir OUTPUTS
file mkdir $dataDir
	
eval "recorder Node    -file $dataDir/GI_Ele_Disp_Profile.txt   		 -time   -nodeRange $p3 [expr $p3+int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2)]  -dof 1 disp"  ; # Records displacement profile of ground improvement element
eval "recorder Node    -file $dataDir/GI_Ele_Accel_Profile.txt   		 -time   -nodeRange $p3 [expr $p3+int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2)]  -dof 1 accel" ; # Records acceleration profile of ground improvement element

eval "recorder Node    -file $dataDir/FF_LeftEdge_Disp_Profile.txt   		 -time   -nodeRange [expr int(($numYnode-1)*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int($numYnode*$numZnode)]  -dof 1 disp"  ; # Records displacement profile of soil at left-side lateral boundary (i.e., midway btw ground improvement elements)
eval "recorder Node    -file $dataDir/FF_LeftEdge_Accel_Profile.txt   		 -time   -nodeRange [expr int(($numYnode-1)*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int($numYnode*$numZnode)]  -dof 1 accel" ; # Records acceleration profile of soil at left-side lateral boundary (i.e., midway btw ground improvement elements)
eval "recorder Node    -file $dataDir/FF_LeftEdge_PWP_Profile.txt   		 -time   -nodeRange [expr int(($numYnode-1)*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int($numYnode*$numZnode)]  -dof 4 vel"   ; # Records pore water ratio profile of soil at left-side lateral boundary (i.e., midway btw ground improvement elements)

eval "recorder Node    -file $dataDir/FF_MiddleEdge_Disp_Profile.txt   		 -time   -nodeRange [expr int(($numXele/2)*$numYnode*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int(($numXele/2)*$numYnode*$numZnode+1+$numZele)]  -dof 1 disp"      ; # Records displacement profile of soil at lateral boundary along plane XZ at distance $S_Y from the center of GI element (i.e., midway btw ground improvement elements)
eval "recorder Node    -file $dataDir/FF_MiddleEdge_Accel_Profile.txt        -time   -nodeRange [expr int(($numXele/2)*$numYnode*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int(($numXele/2)*$numYnode*$numZnode+1+$numZele)]  -dof 1 accel"     ; # Records acceleration profile of soil at lateral boundary along plane XZ at distance $S_Y from the center of GI element (i.e., midway btw ground improvement elements)
eval "recorder Node    -file $dataDir/FF_MiddleEdge_PWP_Profile.txt          -time   -nodeRange [expr int(($numXele/2)*$numYnode*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int(($numXele/2)*$numYnode*$numZnode+1+$numZele)]  -dof 4 vel"     ; # Records pore water ratio profile of soil at lateral boundary along plane XZ at distance $S_Y from the center of GI element (i.e., midway btw ground improvement elements)

eval "recorder Node    -file $dataDir/FF_RightEdge_Disp_Profile.txt   		 -time   -nodeRange [expr int(($numXele)*$numYnode*$numZnode+($numYnode-1)*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int($numXnode*$numYnode*$numZnode)]  -dof 1 disp"      ; # Records displacement profile of soil at right side lateral boundary (i.e., midway btw ground improvement elements)
eval "recorder Node    -file $dataDir/FF_RightEdge_Accel_Profile.txt         -time   -nodeRange [expr int(($numXele)*$numYnode*$numZnode+($numYnode-1)*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int($numXnode*$numYnode*$numZnode)]  -dof 1 accel" 	  ; # Records acceleration profile of soil at at right side lateral boundary  (i.e., midway btw ground improvement elements)
eval "recorder Node    -file $dataDir/FF_RightEdge_PWP_Profile.txt           -time   -nodeRange [expr int(($numXele)*$numYnode*$numZnode+($numYnode-1)*$numZnode+1+($LP-$z0-$z1)/$zSize2)] [expr int($numXnode*$numYnode*$numZnode)]  -dof 4 vel" 	  ; # Records pore water ratio profile of soil at at right side lateral boundary  (i.e., midway btw ground improvement elements)

eval "recorder Element -file $dataDir/ShearStresses_GI_Element.txt 		   	 -time	 -eleRange  [expr int(($numXele/2)*($numYele/2)*$numZele-($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2))] [expr int(($numXele/2)*($numYele/2)*$numZele)]  material 1 stress"  ; # Records stresses in GI element


################################################################################################
#### Rigid Base Boundary Condition - Apply Ground Motion At the Base in the Form of Displacement
################################################################################################

# set FACT  1
# set Gaccel 500

# timeSeries Path $Gaccel -fileTime SCTime.txt -filePath SCDisp.txt  -factor $FACT

# pattern Plain 5000 500 {

# for {set i 1} {$i <= $numXnode} {incr i 1} {
  # for {set j 1} {$j <= $numYnode} {incr j 1} {

        # sp [expr 1+($i-1)*$numYnode*$numZnode+($j-1)*$numZnode] 1 1.0 
		
      # }
    # }  
# }
#### Or you can use the code below to apply the motion. Both give the same results.
##pattern MultipleSupport 4000 {
##                                                                                
##groundMotion 2000 Plain -disp $Gaccel
##
##for {set i 1} {$i <= $numXnode} {incr i 1} {
##  for {set j 1} {$j <= $numYnode} {incr j 1} {
##
##        imposedMotion [expr 1+($i-1)*$numYnode*$numZnode+($j-1)*$numZnode] 1 2000 
##      }
##    }   
##}
########################################################################
#### End of Rigid Base Boundary Condition
########################################################################

########################################################################
#### Compliant Base Boundary Condition - Motion in the form of velocity
########################################################################

model basic -ndm 3 -ndf 3

set ID   200000
set ID_2 400000
set m 0
for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {
    for {set k 1} {$k <= 1} {incr k 1} {

      ### nodes xdim 
	  if { $i <= [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 1]  } {
	     set xdim [expr ($i-1)*$xSize1]
		 } else {
	  if { $i == [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 2]   } {
	     set xdim [expr $S/2.0-$BP/2.0]
		 } else {
	  if { $i > [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 2]  && $i <= [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 6] } {
	     set xdim [expr $S/2.0-$BP/2.0+($i-(($S/2.0-$BP/2.0-$WInf)/$xSize1 + 2))*$BP/4.0]	  
		 } else {
	  if { $i == [expr ($S/2.0-$BP/2.0-$WInf)/$xSize1 + 7]   } {
	     set xdim [expr $S/2.0+$BP/2.0+$WInf]	
		 } else {
         set xdim [expr $S/2.0+$BP/2.0+$WInf + ($i-(($S/2.0-$BP/2.0-$WInf)/$xSize1 + 7))*$xSize1]
		 }}}}
		 
      ### nodes ydim 
	  if { $j <= [expr ($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1 + 1]  } {
	     set ydim [expr ($j-1)*$ySize1]
		 } else {
	  if { $j == [expr ($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1 + 2]   } {
	     set ydim [expr $S_Y/2.0-$BP/2.0]
		 } else {
	  if { $j == [expr ($S_Y/2.0-$BP/2.0-$WInf_Y)/$ySize1 + 3]   } {
	     set ydim [expr $S_Y/2.0-$BP/4.0]	  
		 } else {
           set ydim [expr $S_Y/2.0]	
             }}}

      ### nodes zdim 
      if { $k <= [expr ($z2/$zSize2+1)] } {
          set   zdim [expr ($k-1)*$zSize2]
      } else {
         if { $k > [expr ($z2/$zSize2)+($z1/$zSize1)+1] } {
          set   zdim [expr $z2+$z1+($k-(($z2/$zSize2)+($z1/$zSize1)+1))*$zSize0]
      } else {
          set   zdim [expr $z2+($k-(($z2/$zSize2)+1))*$zSize1]
      }}
	  
       set   nodeNum   [expr $m*$numZnode+$k] 
	  
	   ## Top level
       node  [expr $ID+$m]    $xdim $ydim $zdim
	   ## Bottom level
	   node  [expr $ID_2+$m]  $xdim $ydim $zdim
	   
	   ## Top level
	   fix 	 [expr $ID+$m]    0 1 1 ; #User-defined --- release constraint in other directions if you want to apply motion in different directions.
	   ## Bottom level
	   fix 	 [expr $ID_2+$m]  1 1 1
	   ## Tie top level nodes to original base nodes
	   equalDOF [expr $ID+$m]  $nodeNum   1 2 3

   }
    set m [expr ($m+1)]
  } 
}

# User-Defined --- Material properties below model base
set rockDen 125.0        ;# User-Defined  --- Rock Unit weight
set rockVs  2000.0       ;# User-Defined  --- Rock shear wave velocity
set rockPoisson 0.25    ;# User-Defined  --- Rock Poisson's ratio

# Compressive wave (p-wave) --- In case you need to apply motion in vertical direction
set rockG [expr $rockVs*$rockVs*$rockDen]
set rockM [expr 2*$rockG*(1-$rockPoisson)/(1-2*$rockPoisson)]
set rockVp [expr pow($rockM/$rockDen,0.5)]


# Define Dashpot Material Elements
set m 0
for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {

		uniaxialMaterial Viscous [expr 500000+$m] [expr $rockDen*$rockVs*$xSize1*$ySize1] 1.0
		element zeroLength       [expr 600000+$m]  [expr $ID_2+$m]  [expr $ID+$m] -mat [expr 500000+$m]   -dir  1
		set m [expr ($m+1)]
      }
    } 
	
# load pattern for u-rock 
set FACT  1
set Gaccel 1000
timeSeries Path $Gaccel -fileTime Time.txt -filePath Velocity.txt  -factor $FACT

pattern Plain 300 1000 {
	set m 0
		for {set i 1} {$i <= $numXnode} {incr i 1} {
			for {set j 1} {$j <= $numYnode} {incr j 1} {
				load    [expr $ID+$m]    [expr $rockDen*$rockVs*$xSize1*$ySize1] 0.0 0.0 
				set m [expr ($m+1)]
				}
			}

}

########################################################################
#### End of Compliant Base Boundary Condition
########################################################################

########################################
#### Define Seismic Analysis Algorithms
########################################

set compdT  0.01		; #User-Defined --- Time-step
set EQduration 29.50	; #User-Defined --- Duration of input earthquake motion

#set gamma  0.6
#integrator Newmark  $gamma  [expr pow($gamma+0.5, 2)/4.0]
integrator HHT 0.5 
#numberer RCM
numberer Plain
constraints Penalty 1.0e12 1.0e12
test NormDispIncr 1e-3 50 1
algorithm KrylovNewton
#system SparseSYM
system Mumps
analysis VariableTransient

 for {set numIncr 1} {$numIncr <= [expr int($EQduration/$compdT)] } {incr numIncr 1} {
        puts "##### Seismic excitation step : $numIncr #####";
        analyze 1 $compdT [expr $compdT/100.0] $compdT  20
        }

set endT [clock seconds]
set ElapsedTime [expr $endT-$startT]
set ElapsedHours [expr int($ElapsedTime/3600.)]
set ElapsedMins [expr int($ElapsedTime/60.-$ElapsedHours*60.)]
set ElapsedSecs [expr int($ElapsedTime-$ElapsedHours*3600-$ElapsedMins*60.)]
puts "Execution time: $ElapsedHours hours and $ElapsedMins minutes and $ElapsedSecs seconds !!"

########################################
#### End of the TCL Code
########################################

wipe